//----------------------------------------------------------------------
//This is the basic shader/material definition file for Rosetta Material pipeline
//
//Author - Michal Valient
//Copyright (C) 2004 Caligari corporation
//
//----------------------------------------------------------------------

#ifndef RSD3DMATERIALFROMME_SHADERCORE_FX_DEFINED
#define RSD3DMATERIALFROMME_SHADERCORE_FX_DEFINED

//Shader and material info
#define RsShader_Info(VAR, TYPE, NAME, METHODNAME) \
string VAR : RsD3D_ShaderInfo < \
    string  Rs_Type = #TYPE; \
    string  Rs_Name = #NAME; \
    string  Rs_MethodName = #METHODNAME; \
>; \


//structures
typedef struct {
    RtFloat4    D3_pInputPosition   :   POSITION;
    RtFloat3    D3_vInputNormal     :   NORMAL;
    RtFloat3    D3_vInputTangent    :   TANGENT;
    float2      D3_vInputTexCoords1 :   TEXCOORD0;
    float2      D3_vInputTexCoords2 :   TEXCOORD1;
    RtFloat4    D3_cInputColor      :   COLOR0;
} D3_VERTEXINPUT;

typedef struct {
    RtFloat4    D3_pClipPosition    :   POSITION;
    RtFloat4    D3_vVertexTexCoords :   TEXCOORD0;
    RtFloat4    D3_pVertexPosition  :   TEXCOORD1;
    RtFloat4    D3_vEye             :   TEXCOORD2;
    RtFloat3x3  D3_mToWorld         :   TEXCOORD3;
    RtFloat4    D3_vLightVector     :   TEXCOORD6;
    RtFloat4    D3_cLightColor      :   TEXCOORD7;
    RtFloat4    D3_cVertexColor     :   COLOR0;
} D3_VERTEXOUTPUT;

typedef struct {
    RtFloat4    D3_vVertexTexCoords :   TEXCOORD0;
    RtFloat4    D3_pVertexPosition  :   TEXCOORD1;
    RtFloat4    D3_vEye             :   TEXCOORD2;
    RtFloat3x3  D3_mToWorld         :   TEXCOORD3;
    RtFloat4    D3_vLightVector     :   TEXCOORD6;
    RtFloat4    D3_cLightColor      :   TEXCOORD7;
    RtFloat4    D3_cVertexColor     :   COLOR0;
} D3_PIXELINPUT;

typedef struct {
    float4  D3_cOutColor        :   COLOR0;
} D3_PIXELOUTPUT;

//----------------------------------------------------------------------
//BEGIN - Constant parameters filled by the renderer
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//This parameter is filled by engine with transformation from object space to world space.
uniform RtFloat4x4 D3_mObjectToWorld    :   RsD3DToWorldTransform;

//----------------------------------------------------------------------
//This parameter is filled by engine with transformation from object space to world space for normals.
uniform RtFloat3x3 D3_mObjectToWorldN   :   RsD3DToWorldTransformN;

//----------------------------------------------------------------------
//This parameter is filled by engine with transformation from world space to object space.
uniform RtFloat4x4 D3_mWorldToObject    :   RsD3DToObjectTransform;

//----------------------------------------------------------------------
//This parameter is filled by engine with transformation from object space to camera clip space.
uniform RtFloat4x4 D3_mObjectToClip     :   RsD3DToClipTransform;

//----------------------------------------------------------------------
//This parameter is filled by engine with eye(Camera) position specified in world space.
uniform RtFloat4 D3_pEye                :   RsD3DEyePosition;

//----------------------------------------------------------------------
//END - Constant parameters filled by the renderer
//----------------------------------------------------------------------

//Define all variables needed for the shader input
#define rsxi_VarShaderInput \
RtFloat3 I_vEye; \
RtFloat  I_vEyeDist; \
RtFloat4 I_Pos; \
RtFloat3 I_Coords01; \
RtFloat3 I_Coords02; \
RtFloat4 I_Color; \
RtFloat3 I_Normal; \
RtFloat3 I_Tangent; \
RtFloat3 I_vLight; \
RtFloat  I_vLightDist; \
RtFloat4 I_LightColor; \
RtFloat3x3 I_TangentToWorld

//Define the variables in the function definition
#define rsxi_DefineParamShaderInput \
RtFloat3 I_vEye, \
RtFloat  I_vEyeDist, \
RtFloat4 I_Pos, \
RtFloat3 I_Coords01, \
RtFloat3 I_Coords02, \
RtFloat4 I_Color, \
RtFloat3 I_Normal, \
RtFloat3 I_Tangent, \
RtFloat3 I_vLight, \
RtFloat  I_vLightDist, \
RtFloat4 I_LightColor, \
RtFloat3x3 I_TangentToWorld


//Put variables into the function call
#define rsxi_SendShaderInput \
I_vEye, \
I_vEyeDist, \
I_Pos, \
I_Coords01, \
I_Coords02, \
I_Color, \
I_Normal, \
I_Tangent, \
I_vLight, \
I_vLightDist, \
I_LightColor, \
I_TangentToWorld

void rsxi_Noop() {
}

#define tsxi_GetInputEyePos D3_pEye
#define tsxi_SetInputEyePos(J) rsxi_Noop()

#define tsxi_GetInputEyeDir I_vEye
#define tsxi_SetInputEyeDir(J) I_vEye = J

#define tsxi_GetInputEyeDist I_vEyeDist
#define tsxi_SetInputEyeDist(J) I_vEyeDist = J

#define tsxi_GetInputPosition I_Pos
#define tsxi_SetInputPosition(J) I_Pos = J

#define tsxi_GetInputTexCoords I_Coords01
#define tsxi_SetInputTexCoords(J) I_Coords01 = J

#define tsxi_GetInputTexCoords1 I_Coords01
#define tsxi_SetInputTexCoords1(J) I_Coords01 = J

#define tsxi_GetInputTexCoords2 I_Coords02
#define tsxi_SetInputTexCoords2(J) I_Coords02 = J

#define tsxi_GetInputColor I_Color
#define tsxi_SetInputColor(J) I_Color = J

#define tsxi_GetInputNormal I_Normal
#define tsxi_SetInputNormal(J) I_Normal = J

#define tsxi_GetInputTangent I_Tangent
#define tsxi_SetInputTangent(J) I_Tangent = J

#define tsxi_GetInputAlpha I_Color.a
#define tsxi_SetInputAlpha(J) I_Color.a = J

#define tsxi_GetInputLightDir I_vLight
#define tsxi_SetInputLightDir(J) I_vLight = J

#define tsxi_GetInputLightDist I_vLightDist
#define tsxi_SetInputLightDist(J) I_vLightDist = J

#define tsxi_GetInputLightColor I_LightColor
#define tsxi_SetInputLightColor(J) I_LightColor = J

#define tsxi_GetInputTangentToWorld I_TangentToWorld
#define tsxi_SetInputTangentToWorld(J) I_TangentToWorld = J

#define tsxi_GetInputWorldToObject D3_mWorldToObject
#define tsxi_SetInputWorldToObject(J) rsxi_Noop()

#define tsxi_GetInputObjectToWorld D3_mObjectToWorld
#define tsxi_SetInputObjectToWorld(J) rsxi_Noop()

#endif //RSD3DMATERIALFROMME_SHADERCORE_FX_DEFINED
